import { Injectable } from '@angular/core';
import { Database } from '../database';
import { Socket } from '../socket';
import { Events } from 'ionic-angular';
var Queue = (function () {
    function Queue(database, events, socket) {
        var _this = this;
        this.database = database;
        this.events = events;
        this.socket = socket;
        this.columns = [];
        this.tableName = 'atajoApiQueue';
        this.isProcessing = false;
        this.processInterval = false;
        this.columns =
            [
                {
                    name: 'pid',
                    type: 'VARCHAR(64) PRIMARY KEY NOT NULL'
                }, {
                    name: 'tx',
                    type: 'TEXT'
                }, {
                    name: 'status',
                    type: 'VARCHAR(10)'
                }, {
                    name: 'submitAt',
                    type: 'INT'
                }, {
                    name: 'responseAt',
                    type: 'INT'
                }
            ];
        this.events.subscribe('socket:connect', function () {
            _this.database.create(_this.tableName, _this.columns).then().catch();
            _this.startProcess();
        });
        this.events.subscribe('socket:disconnect', function () {
            _this.stopProcess();
            _this.queueBusy();
        });
        this.events.subscribe('socket:client:rx', function (tx) { _this.transactionResponse(tx); });
    }
    Queue.prototype.add = function (pid, transaction) {
        var _this = this;
        console.log("ADDING TRANSACTION " + pid + " TO QUEUE : ", transaction);
        this.database.create(this.tableName, this.columns).then(function () {
            console.log("TABLE CREATED - CONTINUING");
            transaction.latency.deviceQueuedAt = new Date().getTime();
            var queuedTx = {
                latency: transaction.latency,
                payload: transaction.payload,
                at: transaction.at,
                status: transaction.status,
                lambda: transaction.lambda,
                queue: transaction.queue,
                version: transaction.version,
                environment: transaction.environment
            };
            _this.database.set(_this.tableName, [
                {
                    pid: pid,
                    tx: queuedTx,
                    status: 'QUEUED',
                    submitAt: new Date().getTime()
                }
            ]);
            _this.startProcess();
        });
    };
    Queue.prototype.startProcess = function () {
        var _this = this;
        if (!this.processInterval) {
            this.processInterval = setInterval(function () {
                _this.process();
            }, 20000);
        }
    };
    Queue.prototype.stopProcess = function () {
        clearInterval(this.processInterval);
        this.processInterval = false;
    };
    Queue.prototype.process = function () {
        var _this = this;
        if (this.socket.connected()) {
            console.log("QUEUE: PROCESSING");
            this.list().then(function (queue) {
                console.log("QUEUE IS : ", queue);
                var hadQueued = false;
                var _loop_1 = function (i) {
                    var job = queue[i];
                    if (job.status == 'QUEUED') {
                        hadQueued = true;
                        console.log("PROCESSING " + job.pid);
                        _this.database.update(_this.tableName, [{ column: "status", value: "BUSY" }], [{ column: "pid", op: "=", value: job.pid }]).then(function () {
                            _this.events.publish('queue:update', { pid: job.pid, status: 'BUSY' });
                            var transaction = job.tx;
                            transaction.latency.deviceRequestAt = new Date().getTime();
                            _this.socket.transaction(transaction);
                        }).catch(function (err) { return console.log("COULD NOT UPDATE JOB : ", err); });
                    }
                };
                for (var i in queue) {
                    _loop_1(i);
                }
                if (!hadQueued) {
                    _this.stopProcess();
                }
            }).catch(function (error) { });
        }
        else {
            console.debug("QUEUE: NOT CONNECTED");
            this.stopProcess();
        }
    };
    Queue.prototype.transactionResponse = function (rx) {
        var _this = this;
        if (rx.pid) {
            console.log("QUEUE:CLIENT:RX : ", rx);
            var pid_1 = rx.pid;
            this.database.get(this.tableName, "pid='" + pid_1 + "'").then(function (transaction) {
                if (transaction) {
                    //CALC PERCENTAGES FROM CHUNKS
                    var chunkId = rx.response.chunkId + 1;
                    var chunkTotal = rx.response.chunkTotal;
                    var progress_1 = (100 / chunkTotal) * chunkId;
                    progress_1 = isNaN(progress_1) ? 0 : progress_1;
                    if (!rx.response.data)
                        rx.response.data = [];
                    if (Array.isArray(rx.response.data)) {
                        if (rx.response.data.length == 1) {
                            rx.response.data = rx.response.data[0];
                        }
                    }
                    if (rx.error) {
                        _this.database.update(_this.tableName, [{ column: "status", value: "FAILED" }], [{ column: "pid", op: "=", value: pid_1 }]).then(function () {
                            _this.events.publish('queue:update', { error: 1, progress: progress_1, pid: pid_1, data: rx.response.data });
                        });
                    }
                    else {
                        _this.database.update(_this.tableName, [{ column: "status", value: "DONE" }], [{ column: "pid", op: "=", value: pid_1 }]).then(function () {
                            _this.events.publish('queue:update', { error: 0, progress: progress_1, pid: pid_1, data: rx.response.data });
                        });
                    }
                }
                else {
                    console.warn("REALTIME TRANSACTION FOR " + pid_1 + " NOT CACHED - POSSIBLY QUEUED");
                }
            });
        }
        else
            return console.error("SOCKET:TX_RESPONSE INVALID : ", rx);
    };
    Queue.prototype.list = function (status) {
        if (status)
            return this.database.get(this.tableName, "status='" + status.toUpperCase() + "'");
        return this.database.get(this.tableName);
    };
    Queue.prototype.queueBusy = function () {
        console.log("QUEUE.FAILBUSY");
        this.database.update(this.tableName, [{ column: "status", value: "QUEUED" }], [{ column: "status", op: "=", value: "BUSY" }]);
    };
    Queue.prototype.retryFailed = function () {
        console.log("QUEUE.RETRYFAILED");
        this.database.update(this.tableName, [{ column: "status", value: "QUEUED" }], [{ column: "status", op: "=", value: "FAILED" }]);
        this.startProcess();
    };
    Queue.prototype.delete = function (pid) {
    };
    Queue.prototype.clear = function () {
        console.log("QUEUE.RETRYFAILED");
        this.database.nuke(this.tableName);
        this.stopProcess();
    };
    Queue.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    Queue.ctorParameters = function () { return [
        { type: Database, },
        { type: Events, },
        { type: Socket, },
    ]; };
    return Queue;
}());
export { Queue };
//# sourceMappingURL=index.js.map