import { Injectable } from '@angular/core';
import { SQLite, SQLiteObject } from '@ionic-native/sqlite';

@Injectable()
export class Sqlite {

  private db;

  constructor(private sqlite: SQLite) {

  }

  public init() {

    console.log("SQLite INIT CALLED");
    this.sqlite.create({
      name: 'atajo.client.db',
      location: 'default'
    })
      .then((db: SQLiteObject) => {
        console.log("SQLite INIT DONE");
        this.db = db;
      })
      .catch(error => console.log("SQLite ERROR : ", error));
  }

  private isReady() {
    return new Promise((resolve, reject) => {
      if (!this.db) {
        this.init();
        let interval = setInterval(() => {
          if (this.db) {
            resolve();
            clearInterval(interval);
          }
        }, 10);
      } else {
        resolve();
      }
    });
  }

  public create(table, columns) {
    return new Promise((resolve, reject) => {
      console.log("SQLite CREATE CALLED : " + table + " -> ", columns);
      this.isReady().then(() => {
        console.log("SQLite READY...");

        let columnsWithTypes = [];
        columns.forEach((column) => {
          columnsWithTypes.push(column.name + ' ' + column.type);
        });
        let query = 'CREATE TABLE IF NOT EXISTS ' + table + ' (' + columnsWithTypes.join(', ') + ')';
        console.log("SQLite EXECUTING : ", query);
        this.db.executeSql(query, {})
          .then(() => {
            resolve();
          })
          .catch(error => {
            console.error(error);
            reject();
          });

      });

    });
  }

  public set(table, values) {
    return new Promise((resolve, reject) => {

      this.isReady().then(() => {
        let promises = [];
        values.forEach((value) => {
          let rowColumns = [];
          let rowValues = [];
          let valuePlaceholders = [];
          for (let v in value) {
            if (value.hasOwnProperty(v)) {
              rowColumns.push(v);
              rowValues.push(JSON.stringify(value[v]));
            }
          }
          rowColumns.forEach(() => {
            valuePlaceholders.push('?');
          });
          let query = 'INSERT INTO ' + table + ' (' + rowColumns.join(', ') + ') VALUES (' + valuePlaceholders.join(', ') + ')';
          console.log("SQLite EXECUTING : ", query);
          console.log("SQLite WITH VALUES  : ", rowValues);
          promises.push(this.db.executeSql(query, rowValues));
        });

        Promise.all(promises).then(() => {
          resolve();
        })
          .catch(error => {
            console.error(error);
            reject();
          });
      });

    });
  }

  public get(table, where?) {
    return new Promise((resolve, reject) => {
      this.isReady().then(() => {
        let whereStatement = (typeof where !== "undefined") ? 'WHERE ' + where : '';

        this.db.executeSql('SELECT * FROM ' + table + ' ' + whereStatement, {})
          .then((data) => {

            let rows = [];
            let len = data.rows.length;
            for (let i = 0; i < len; ++i) {
              let obj = data.rows.item(i);
              let processedRow = {};
              for (let r in obj) {
                if (obj.hasOwnProperty(r)) {
                  processedRow[r] = JSON.parse(obj[r]);
                }
              }
              rows.push(processedRow);
            }

            resolve(rows);
          })
          .catch(error => {
            console.error(error);
            reject();
          });
      });
    });
  }

  public update(table, values, where?) {
    return new Promise((resolve, reject) => {
      this.isReady().then(() => {

        let setColumns = [];
        let whereColumns = [];
        let rowValues = [];
        let valuePlaceholders = [];

        values.forEach((value) => {
          setColumns.push(value.column + '=?');
          rowValues.push(JSON.stringify(value.value));
        });

        if (where) {
          where.forEach((value) => {
            whereColumns.push(value.column + (value.op || '=') + '?');
            rowValues.push(JSON.stringify(value.value));
          });
        }

        let whereStatement = (typeof where !== "undefined") ? 'WHERE ' + whereColumns.join(', ') : '';
        let query = 'UPDATE ' + table + ' SET ' + setColumns.join(', ') + ' ' + whereStatement + '';
        console.log("UPDATE QUERY : ", query);
        this.db.executeSql(query, rowValues).then(result => {
          resolve(result);
        }).catch(error => {
          console.error(error);
          reject();
        });
      })
    });

  }

  public remove(table, where?) {
    return new Promise((resolve, reject) => {
      this.isReady().then(() => {
        let whereStatement = (typeof where !== "undefined") ? 'WHERE ' + where : '';
        this.db.executeSql('DELETE FROM ' + table + ' ' + whereStatement, {})
          .then(() => {
            resolve();
          })
          .catch(error => {
            console.error(error);
            reject();
          });
      });
    });
  }

  public nuke(table) {
    return new Promise((resolve, reject) => {
      this.isReady().then(() => {
        this.db.executeSql('DROP TABLE IF EXISTS ' + table, {})
          .then(() => {
            resolve();
          })
          .catch(error => {
            console.error(error);
            reject();
          });
      });
    });
  }

}
