import { Injectable } from '@angular/core';
import { Database } from '../database';
import { Socket } from '../socket';
import { Events } from 'ionic-angular';

@Injectable()
export class Queue {

    columns = [];
    tableName = 'atajoApiQueue';
    isProcessing = false;
    processInterval: any = false;

    constructor(private database: Database, private events: Events, private socket: Socket) {

        this.columns =
            [
                {
                    name: 'pid',
                    type: 'VARCHAR(64) PRIMARY KEY NOT NULL'
                }, {
                    name: 'tx',
                    type: 'TEXT'
                }, {
                    name: 'status',
                    type: 'VARCHAR(10)'
                }, {
                    name: 'submitAt',
                    type: 'INT'
                }, {
                    name: 'responseAt',
                    type: 'INT'
                }
            ];

        this.database.create(this.tableName, this.columns).then(() => {
            console.log("TABLE CREATED - CONTINUING");
        }, () => {
            console.error("TABLE " + this.tableName + " NOT CREATED");
        });


        this.events.subscribe('socket:connect', () => {

            this.database.create(this.tableName, this.columns).then().catch();
            this.startProcess();


        });

        this.events.subscribe('socket:disconnect', () => {

            this.stopProcess();
            this.queueBusy();

        });



        this.events.subscribe('socket:client:rx', (tx) => { this.transactionResponse(tx); });


    }

    add(pid: String, transaction) {

        console.log("ADDING TRANSACTION " + pid + " TO QUEUE : ", transaction);

        transaction.latency.deviceQueuedAt = new Date().getTime();

        let queuedTx = {
            latency: transaction.latency,
            payload: transaction.payload,
            at: transaction.at,
            status: transaction.status,
            lambda: transaction.lambda,
            queue: transaction.queue,
            version: transaction.version,
            environment: transaction.environment
        };


        this.database.set(this.tableName, [
            {
                pid: pid,
                tx: queuedTx,
                status: 'QUEUED',
                submitAt: new Date().getTime()
            }
        ]);

        this.startProcess();



    }

    private startProcess() {

        if (!this.processInterval) {
            this.processInterval = setInterval(() => {

                this.process();

            }, 20000);
        }

    }

    private stopProcess() {

        clearInterval(this.processInterval);
        this.processInterval = false;

    }

    private process() {

        if (this.socket.connected()) {

            console.log("QUEUE: PROCESSING");

            this.list().then(queue => {

                console.log("QUEUE IS : ", queue);

                let hadQueued = false;

                for (let i in queue) {
                    let job = queue[i];
                    if (job.status == 'QUEUED') {

                        hadQueued = true;
                        console.log("PROCESSING " + job.pid);

                        this.database.update(this.tableName, [{ column: "status", value: "BUSY" }], [{ column: "pid", op: "=", value: job.pid }]).then(() => {

                            this.events.publish('queue:update', { pid: job.pid, status: 'BUSY' });
                            let transaction = job.tx;
                            transaction.latency.deviceRequestAt = new Date().getTime();
                            this.socket.transaction(transaction);

                        }).catch(err => console.log("COULD NOT UPDATE JOB : ", err));
                    }

                }

                if (!hadQueued) {
                    this.stopProcess();
                }


            }).catch(error => { });

        } else {
            console.debug("QUEUE: NOT CONNECTED");
            this.stopProcess();
        }




    }

    private transactionResponse(rx) {

        if (rx.pid) {

            console.log("QUEUE:CLIENT:RX : ", rx);
            let pid = rx.pid;
            this.database.get(this.tableName, "pid='" + pid + "'").then(transaction => {

                if (transaction) {

                    //CALC PERCENTAGES FROM CHUNKS
                    let chunkId = rx.response.chunkId + 1;
                    let chunkTotal = rx.response.chunkTotal;
                    let progress = (100 / chunkTotal) * chunkId;

                    progress = isNaN(progress) ? 0 : progress;

                    if (!rx.response.data) rx.response.data = [];

                    if (Array.isArray(rx.response.data)) {
                        if (rx.response.data.length == 1) {
                            rx.response.data = rx.response.data[0];
                        }
                    }

                    if (rx.error) {
                        this.database.update(this.tableName, [{ column: "status", value: "FAILED" }], [{ column: "pid", op: "=", value: pid }]).then(() => {
                            this.events.publish('queue:update', { error: 1, progress: progress, pid: pid, data: rx.response.data });
                        });

                    } else {
                        this.database.update(this.tableName, [{ column: "status", value: "DONE" }], [{ column: "pid", op: "=", value: pid }]).then(() => {
                            this.events.publish('queue:update', { error: 0, progress: progress, pid: pid, data: rx.response.data });
                        });
                    }


                }
                else {
                    console.warn("REALTIME TRANSACTION FOR " + pid + " NOT CACHED - POSSIBLY QUEUED");
                }



            })



        }
        else return console.error("SOCKET:TX_RESPONSE INVALID : ", rx);



    }


    public list(status?) {
        if (status) return this.database.get(this.tableName, "status='" + status.toUpperCase() + "'");
        return this.database.get(this.tableName);
    }

    private queueBusy() {
        console.log("QUEUE.FAILBUSY");
        this.database.update(this.tableName, [{ column: "status", value: "QUEUED" }], [{ column: "status", op: "=", value: "BUSY" }]);
    }

    public retryFailed() {
        console.log("QUEUE.RETRYFAILED");
        this.database.update(this.tableName, [{ column: "status", value: "QUEUED" }], [{ column: "status", op: "=", value: "FAILED" }]);
        this.startProcess();
    }

    public delete(pid: string) {

    }

    public clear() {
        console.log("QUEUE.RETRYFAILED");
        this.database.nuke(this.tableName);
        this.stopProcess();
    }

}